//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Device/DatafileItem.h
//! @brief     Defines class DatafileItem.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_DEVICE_DATAFILEITEM_H
#define BORNAGAIN_GUI_MODEL_DEVICE_DATAFILEITEM_H

#include <QObject>
#include <QXmlStreamReader>
#include <QXmlStreamWriter>

class Data1DItem;
class Data2DItem;
class DataItem;
class Datafield;
class InstrumentItem;
class MessageService;

//! Provides access to experimental data, for display and fitting.

class DatafileItem : public QObject {
    Q_OBJECT
public:
    DatafileItem();
    DatafileItem(const QString& name, const Datafield& df);
    ~DatafileItem();

    DatafileItem* clone() const;

    //! The name which is presented to the user
    QString realItemName() const { return m_name; }
    void setDatafileItemName(const QString& name);

    //... Data

    DataItem* dataItem() const;
    Data2DItem* data2DItem() const;
    Data1DItem* data1DItem() const;

    size_t rank() const;

    bool holdsDimensionalData() const;

    //... Instrument

    QString instrumentId() const { return m_instrument_id; }
    void setInstrumentId(const QString& id);
    void linkToInstrument(const InstrumentItem* instrument);
    void unlinkFromInstrument();

    //! Returns true, if rotation will affect linked instrument or mask presence.
    bool rotationAffectsSetup() const;
    void rotateData();

    //... Write/read

    void writeTo(QXmlStreamWriter* w) const;
    void readFrom(QXmlStreamReader* r);

    void writeDatafiles(const QString& projectDir) const;
    QString readDatafiles(const QString& projectDir, MessageService* messageService);

    //... Other

    //! Returns the shape of underlying data item
    std::vector<int> axdims() const;

private:
    //! This does not set the link to the instrument! Use linkToInstrument() if you want
    //! to link to this instrument.
    void updateToInstrument(const InstrumentItem* instrument);
    void updateFileName();

    QString m_instrument_id;
    QString m_name;

    std::unique_ptr<DataItem> m_data_item; //!< Can be 1D or 2D item.
};

#endif // BORNAGAIN_GUI_MODEL_DEVICE_DATAFILEITEM_H
