//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Files/DatafilesSet.h
//! @brief     Defines class DatafilesSet.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_FILES_DATAFILESSET_H
#define BORNAGAIN_GUI_MODEL_FILES_DATAFILESSET_H

#include "GUI/Model/Files/DatafilesCleaner.h"
#include <QObject>
#include <QXmlStreamWriter>

class DataItem;
class DatafileItem;
class InstrumentsSet;
class MessageService;

//! The DatafilesSet class is a model to store all imported DatafileItem's.
class DatafilesSet : public QObject {
    Q_OBJECT
public:
    explicit DatafilesSet();
    ~DatafilesSet();

    QVector<DataItem*> dataItems() const;
    void clear();

    void writeTo(QXmlStreamWriter* w) const;
    void readFrom(QXmlStreamReader* r);

    void writeDatafiles(const QString& projectDir);
    void readDatafiles(const QString& projectDir, MessageService* messageService);

    void insertDatafileItem(DatafileItem* dfi);
    QVector<DatafileItem*> dfileItems() const;
    QVector<DatafileItem*> dfileItems(int rank) const;

    QVector<DatafileItem*> realItems1D() const;
    QVector<DatafileItem*> realItems2D() const;

    void removeDatafileItem(DatafileItem* realItem);

    QStringList realItemNames() const;

    int currentIndex() const { return m_current_index; }
    void setCurrentIndex(int index) { m_current_index = index; }

    int selectedRank() const { return m_selected_rank; }
    void setSelectedRank(int rank) { m_selected_rank = rank; }

private:
    DatafilesCleaner dataFilesCleaner;
    std::vector<std::unique_ptr<DatafileItem>> m_dfile_items;
    int m_current_index = -1;
    int m_selected_rank = -1;
};

#endif // BORNAGAIN_GUI_MODEL_FILES_DATAFILESSET_H
