//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Tune/JobsSet.h
//! @brief     Defines class JobsSet.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_TUNE_JOBSSET_H
#define BORNAGAIN_GUI_MODEL_TUNE_JOBSSET_H

#include "Base/Types/OwningVector.h"
#include "GUI/Model/Files/DatafilesCleaner.h"
#include <QMap>
#include <QObject>
#include <QXmlStreamReader>
#include <QXmlStreamWriter>

class DataItem;
class DatafileItem;
class ISimulation;
class InstrumentItem;
class JobItem;
class JobWorker;
class MessageService;
class SampleItem;
class SimulationOptionsItem;

class JobsSet : public QObject {
    Q_OBJECT
public:
    explicit JobsSet(QObject* parent = nullptr);
    ~JobsSet() override;

    void writeTo(QXmlStreamWriter* w) const;
    void readFrom(QXmlStreamReader* r);

    void writeDatafiles(const QString& projectDir);
    void readDatafiles(const QString& projectDir, MessageService* messageService);

    JobItem* createJobItem();
    void addJobItem(JobItem* job_item);

    QVector<JobItem*> jobItems() const;
    QVector<const JobItem*> cJobItems() const;

    void restoreBackupPars(JobItem* job_item, int index);

    bool hasUnfinishedJobs() const;

    QVector<DataItem*> dataItems() const;

    void runJob(JobItem* job_item);
    void cancelJob(JobItem* job_item);
    void removeJob(JobItem* job_item);

    int currentJobIndex() const { return m_current_index; }
    void setJobIndex(int index) { m_current_index = index; }

signals:
    void jobMeritsAttention(JobItem* item);
    void globalProgress(int);
    void jobAdded();

private slots:
    void onFinishedJob(JobItem* job_item);
    void onProgressUpdate();
    void onCancelAllJobs();

private:
    QString generateJobName() const;

    DatafilesCleaner dataFilesCleaner;
    OwningVector<JobItem> m_job_items;
    int m_current_index = -1;
};

#endif // BORNAGAIN_GUI_MODEL_TUNE_JOBSSET_H
